@@ -27,6 +27,7 @@ from typing import (
 )
 
 import psycopg2
+import psycopg2.extensions
 from packaging.version import Version
 from psycopg2 import DatabaseError
 from psycopg2.errors import InvalidSchemaName, UndefinedTable
@@ -83,6 +84,8 @@ _API_VERSION = "0.0.1"
 # footprint more predictable. We hence chunk some heavy queries like object
 # index uploading to allow registering multiple objects at the same time whilst
 # also limiting the query's maximum size.
+
+# TODO this breaks e.g. object index with a wide table without a PK
 API_MAX_QUERY_LENGTH = 261000
 
 # In addition, some API calls (like get_object_meta) allow variadic arguments
@@ -213,6 +216,10 @@ class PsycopgEngine(SQLEngine):
         """
         super().__init__()
 
+        # Allow users to send SIGINT to quickly terminate sgr (instead of waiting for a PG
+        # statement to finish)
+        psycopg2.extensions.set_wait_callback(psycopg2.extras.wait_select)
+
         if not conn_params and not pool:
             raise ValueError("One of conn_params/pool must be specified!")
 
@@ -307,6 +314,22 @@ class PsycopgEngine(SQLEngine):
             SQL("LOCK TABLE {}.{} IN EXCLUSIVE MODE").format(Identifier(schema), Identifier(table))
         )
 
+    @contextmanager
+    def copy_cursor(self):
+        """
+        Return a cursor that can be used for copy_expert operations
+        """
+
+        # Suspend the callback that terminates the connection on Ctrl+C since it doesn't work
+        # with copy_expert: https://stackoverflow.com/a/37606536
+        thread = psycopg2.extensions.get_wait_callback()
+        try:
+            psycopg2.extensions.set_wait_callback(None)
+            with self.connection.cursor() as cur:
+                yield cur
+        finally:
+            psycopg2.extensions.set_wait_callback(thread)
+
     @property
     def splitgraph_version(self) -> Optional[str]:
         """Returns the version of the Splitgraph library installed on the engine
@@ -1325,7 +1348,7 @@ class PostgresEngine(AuditTriggerChangeEngine, ObjectEngine):
             )
 
             stream = BytesIO()
-            with self.connection.cursor() as cur:
+            with self.copy_cursor() as cur:
                 cur.copy_expert(
                     SQL("COPY {}.{} TO STDOUT WITH (FORMAT 'binary')").format(
                         Identifier(SPLITGRAPH_META_SCHEMA), Identifier(object_id)
@@ -1333,7 +1356,7 @@ class PostgresEngine(AuditTriggerChangeEngine, ObjectEngine):
                     stream,
                 )
             stream.seek(0)
-            with remote_engine.connection.cursor() as cur:
+            with remote_engine.copy_cursor() as cur:
                 cur.copy_expert(
                     SQL("COPY {}.{} FROM STDIN WITH (FORMAT 'binary')").format(
                         Identifier(SPLITGRAPH_META_SCHEMA), Identifier(object_id)
